@@ -8,6 +8,7 @@ use crate::colors::parse_color;
 
 mod colors;
 pub mod plugin;
+mod shapes;
 
 #[derive(Default)]
 struct GlobalSettings {
@@ -34,7 +35,7 @@ impl Controls {
             Some(control) => control
         };
 
-        return result;
+        result
     }
 }
 
@@ -53,7 +54,7 @@ impl Fields {
             Some(control) => control
         };
 
-        return result;
+        result
     }
 }
 
@@ -76,7 +77,7 @@ pub fn instantiate_controls_from_file(filename: &str) -> Controls {
     let lines = read_ui_file(filename);
     let controls = instantiate_controls(lines);
 
-    return controls;
+    controls
 }
 
 pub fn read_ui_file(ui_filename: &str) -> Vec<String> {
@@ -92,7 +93,7 @@ pub fn read_ui_file(ui_filename: &str) -> Vec<String> {
         list.push(line);
     }
 
-    return list;
+    list
 }
 
 pub fn instantiate_controls(lines: Vec<String>) -> Controls {
@@ -157,6 +158,8 @@ pub fn instantiate_controls(lines: Vec<String>) -> Controls {
                 control.fields.insert("texture_name_active".to_string(), "".to_string());
                 control.fields.insert("texture_name_disabled".to_string(), "".to_string());
                 control.fields.insert("on_click_sound".to_string(), "".to_string());
+                control.fields.insert("bounding_box".to_string(), "0;0;0;0".to_string());
+                control.fields.insert("bounding_circle".to_string(), "0;0;0".to_string());
                 control.fields.insert("draw_order".to_string(), "0.0".to_string());
                 control.fields.insert("top_left_position".to_string(), "".to_string());
                 control.fields.insert("center_position".to_string(), "".to_string());
@@ -205,11 +208,10 @@ pub fn instantiate_controls(lines: Vec<String>) -> Controls {
 
     let result = Controls { map: controls };
 
-    return result;
+    result
 }
 
 pub fn spawn_controls(commands: &mut Commands, asset_server: Res<AssetServer>, mut materials: ResMut<Assets<ColorMaterial>>, controls: Controls, screen_size: Vec2, control_group_name: String) -> Vec<Entity> {
-
     let mut results = Vec::new();
     let controls_map = &controls.map;
     for (_, control) in controls_map {
@@ -217,28 +219,28 @@ pub fn spawn_controls(commands: &mut Commands, asset_server: Res<AssetServer>, m
         let size = parse_vec2(control.fields.get_by_name("size"));
         let top_left_position = calculate_top_left_position(control, &controls, screen_size);
 
-        match control.control_type {
-            ControlType::PictureBox => {
-                let entity = spawn_picture_box(top_left_position, size, control, &asset_server, &mut materials, commands, &control_group_name);
+        let entity = spawn_control(control, top_left_position, size, &asset_server, &mut materials, commands, &control_group_name, screen_size);
 
-                results.push(entity);
-            },
-            ControlType::Label => {
-                let entity = spawn_label(top_left_position, screen_size, control, &asset_server, size, commands, &control_group_name);
-
-                results.push(entity);
-            },
-            ControlType::Button => {
-                let entity = spawn_button(top_left_position, size, control, &asset_server, &mut materials, commands, &control_group_name);
-
-                results.push(entity);
-            }
-        }
-
-        //results.push(entity);
+        results.push(entity);
     }
 
-    return results;
+    results
+}
+
+fn spawn_control(control: &Control, top_left_position: Vec2, size: Vec2, asset_server: &Res<AssetServer>, materials: &mut ResMut<Assets<ColorMaterial>>, commands: &mut Commands, control_group_name: &String, screen_size: Vec2) -> Entity {
+    let entity = match control.control_type {
+        ControlType::PictureBox => {
+            spawn_picture_box(top_left_position, size, control, asset_server, materials, commands, control_group_name)
+        },
+        ControlType::Label => {
+            spawn_label(top_left_position, screen_size, control, asset_server, size, commands, control_group_name)
+        },
+        ControlType::Button => {
+            spawn_button(top_left_position, size, control, asset_server, materials, commands, control_group_name)
+        }
+    };
+
+    entity
 }
 
 fn spawn_picture_box(top_left_position: Vec2, size: Vec2, control: &Control, asset_server: &Res<AssetServer>, materials: &mut ResMut<Assets<ColorMaterial>>, commands: &mut Commands, control_group_name: &String) -> Entity {
@@ -253,15 +255,11 @@ fn spawn_picture_box(top_left_position: Vec2, size: Vec2, control: &Control, ass
         .insert(GergControl { group_name: control_group_name.clone() })
         .id();
 
-    println!("Spawned: {}, {}", control.fields.get_by_name("name").clone(), control_group_name.clone());
-
-    return entity;
+    entity
 }
 
 fn spawn_label(top_left_position: Vec2, screen_size: Vec2, control: &Control, asset_server: &Res<AssetServer>, size: Vec2, commands: &mut Commands, control_group_name: &String) -> Entity {
     let top_left_position = Vec2::new(top_left_position.x + screen_size.x * 0.5, screen_size.y * 0.5 - top_left_position.y);
-    //top_left_position.x += screen_size.x * 0.5;
-    //top_left_position.y = screen_size.y * 0.5 - top_left_position.y;
     let min_size = Vec2::new(0.0, 0.0);
     let text = control.fields.get_by_name("text_string");
     let font_handle: Handle<Font> = asset_server.load(format!("fonts/{}", control.fields.get_by_name("font_name")).as_str());
@@ -274,14 +272,14 @@ fn spawn_label(top_left_position: Vec2, screen_size: Vec2, control: &Control, as
         .insert(GergControl { group_name: control_group_name.clone() })
         .id();
 
-    println!("Spawned: {}, {}", control.fields.get_by_name("name").clone(), control_group_name.clone());
-
-    return entity;
+    entity
 }
 
 fn spawn_button(top_left_position: Vec2, size: Vec2, control: &Control, asset_server: &Res<AssetServer>, materials: &mut ResMut<Assets<ColorMaterial>>, commands: &mut Commands, control_group_name: &String) -> Entity {
     let center_position = Vec3::new(top_left_position.x + size.x * 0.5, top_left_position.y - size.y * 0.5, parse_f32(control.fields.get_by_name("draw_order")));
     let scale = Vec3::new(1.0, 1.0, 1.0);
+    let bounding_box = parse_vec4(control.fields.get_by_name("bounding_box"));
+    let bounding_circle = parse_vec3(control.fields.get_by_name("bounding_circle"));
     let texture_name_normal = control.fields.get_by_name("texture_name_normal");
     let mut texture_name_hover = control.fields.get_by_name("texture_name_hover");
     if texture_name_hover.is_empty() {
@@ -310,14 +308,14 @@ fn spawn_button(top_left_position: Vec2, size: Vec2, control: &Control, asset_se
             color_material_handle_hover: color_material_handle_hover,
             color_material_handle_active: color_material_handle_active,
             color_material_handle_disabled: color_material_handle_disabled,
-            on_click_sound: on_click_sound.to_string()
+            on_click_sound: on_click_sound.to_string(),
+            bounding_box,
+            bounding_circle
         })
         .insert(GergControl { group_name: control_group_name.clone() })
         .id();
 
-    println!("Spawned: {}, {}", control.fields.get_by_name("name").clone(), control_group_name.clone());
-
-    return entity;
+    entity
 }
 
 fn get_color_material_handle(path: &str, asset_server: &Res<AssetServer>, control: &Control, materials: &mut ResMut<Assets<ColorMaterial>>) -> Handle<ColorMaterial> {
@@ -329,7 +327,7 @@ fn get_color_material_handle(path: &str, asset_server: &Res<AssetServer>, contro
     }
     let color_material_handle = materials.add(color_material);
     
-    return color_material_handle;
+    color_material_handle
 }
 
 fn to_bevy_color(color_u32: u32) -> Color {
@@ -341,47 +339,52 @@ fn to_bevy_color(color_u32: u32) -> Color {
 
     let color = Color::Rgba { red: r, green: g, blue: b, alpha: a};
 
-    return color;
+    color
 }
 
 fn calculate_top_left_position(control: &Control, controls: &Controls, screen_size: Vec2) -> Vec2 {
-
     let control_size = parse_vec2(control.fields.get_by_name("size"));
     
     let control_dock_with = control.fields.get_by_name("dock_with");
-    if control_dock_with.is_empty() {
-        let top_left_position = control.fields.get_by_name("top_left_position");
+    let tlp = if control_dock_with.is_empty() {
+        calculate_top_left_position_without_docking(control, control_size)
+    } else {
+        calculate_top_left_position_with_docking(control_dock_with, controls, screen_size, control_size, control)
+    };
 
-        if top_left_position.is_empty() {
-            let center_position = control.fields.get_by_name("center_position");
+    tlp
+}
 
-            if center_position.is_empty() {
-                return Vec2::new(0.0 - control_size.x * 0.5, 0.0 + control_size.y * 0.5);
-            } else {
-                let cp = parse_vec2(&center_position);
-                let tlp = Vec2::new(cp.x - control_size.x * 0.5, cp.y + control_size.y * 0.5);
-                
-                return tlp;
-            }
+fn calculate_top_left_position_without_docking(control: &Control, control_size: Vec2) -> Vec2 {
+    let top_left_position = control.fields.get_by_name("top_left_position");
+    let result = if top_left_position.is_empty() {
+        let center_position = control.fields.get_by_name("center_position");
+
+        if center_position.is_empty() {
+            Vec2::new(0.0 - control_size.x * 0.5, 0.0 + control_size.y * 0.5)
         } else {
-            return parse_vec2(&top_left_position);
+            let cp = parse_vec2(&center_position);
+            let tlp = Vec2::new(cp.x - control_size.x * 0.5, cp.y + control_size.y * 0.5);
+        
+            tlp
         }
-    }
+    } else {
+        parse_vec2(&top_left_position)
+    };
 
+    result
+}
+
+fn calculate_top_left_position_with_docking(control_dock_with: &String, controls: &Controls, screen_size: Vec2, control_size: Vec2, control: &Control) -> Vec2 {
     let split = control_dock_with.split("<->").collect::<Vec<&str>>();
     let dock_this = split[1];
     let dock_to = split[0];
-
     let split = dock_to.split(".").collect::<Vec<&str>>();
     let control_to_use_for_docking = split[0];
     let point_on_control_to_anchor_to = split[1];
-
     let pixel1 = get_point_to_dock_to(controls, control_to_use_for_docking, point_on_control_to_anchor_to, screen_size);
-    //println!("Pixel1: {}", pixel1);
-
     let split = dock_this.split(".").collect::<Vec<&str>>();
     let point_on_this_control_to_anchor_to = split[1];
-
     let pixel2 = match point_on_this_control_to_anchor_to.to_lowercase().as_str() {
         "top_left" => Vec2::new(pixel1.x, pixel1.y),
         "center_left" => Vec2::new(pixel1.x, pixel1.y + control_size.y * 0.5),
@@ -397,18 +400,15 @@ fn calculate_top_left_position(control: &Control, controls: &Controls, screen_si
 
         _ => panic!("{} is not implemented.", point_on_this_control_to_anchor_to)
     };
-    //println!("Pixel2: {}", result_pixel);
-
     let offset = control.fields.get_by_name("offset");
     let result_pixel = pixel2 + parse_vec2(offset);
 
-    return result_pixel;
+    result_pixel
 }
 
 fn get_point_to_dock_to(controls: &Controls, control_to_use_for_docking: &str, point_on_control_to_anchor_to: &str, screen_size: Vec2) -> Vec2 {
-
-    if control_to_use_for_docking.to_lowercase() == "screen" {
-        let p = match point_on_control_to_anchor_to.to_lowercase().as_str() {
+    let result = if control_to_use_for_docking.to_lowercase() == "screen" {
+        let point = match point_on_control_to_anchor_to.to_lowercase().as_str() {
             "top_left" => Vec2::new(-screen_size.x * 0.5, screen_size.y * 0.5),
             "center_left" => Vec2::new(-screen_size.x * 0.5, 0.0),
             "bottom_left" => Vec2::new(-screen_size.x * 0.5, -screen_size.y * 0.5),
@@ -424,67 +424,100 @@ fn get_point_to_dock_to(controls: &Controls, control_to_use_for_docking: &str, p
             _ => panic!("{} is not implemented.", point_on_control_to_anchor_to)
         };
     
-        return p;
-    }
+        point
+    } else {
+        let control_to_dock_to = controls.get_by_name(control_to_use_for_docking.to_string());
+        let control_to_dock_to_size = parse_vec2(control_to_dock_to.fields.get_by_name("size"));
+        let parent_top_left_position = calculate_top_left_position(control_to_dock_to, controls, screen_size);
+        let point = match point_on_control_to_anchor_to.to_lowercase().as_str() {
+            "top_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y),
+            "center_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
+            "bottom_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y - control_to_dock_to_size.y),
+        
+            "top_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y),
+            "center_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
+            "bottom_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y - control_to_dock_to_size.y),
 
-    let control_to_dock_to = controls.get_by_name(control_to_use_for_docking.to_string());
-    let control_to_dock_to_size = parse_vec2(control_to_dock_to.fields.get_by_name("size"));
-    let parent_top_left_position = calculate_top_left_position(control_to_dock_to, controls, screen_size);
-    let p = match point_on_control_to_anchor_to.to_lowercase().as_str() {
-        "top_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y),
-        "center_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
-        "bottom_left" => Vec2::new(parent_top_left_position.x, parent_top_left_position.y - control_to_dock_to_size.y),
-    
-        "top_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y),
-        "center_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
-        "bottom_middle" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x * 0.5, parent_top_left_position.y - control_to_dock_to_size.y),
+            "top_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y),
+            "center_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
+            "bottom_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y - control_to_dock_to_size.y),
 
-        "top_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y),
-        "center_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y - control_to_dock_to_size.y * 0.5),
-        "bottom_right" => Vec2::new(parent_top_left_position.x + control_to_dock_to_size.x, parent_top_left_position.y - control_to_dock_to_size.y),
+            _ => panic!("{} is not implemented.", point_on_control_to_anchor_to)
+        };
 
-        _ => panic!("{} is not implemented.", point_on_control_to_anchor_to)
+        point
     };
 
-    return p;
+    result
 }
 
 fn get_string(s: String) -> String {
-
     let right_side_of_colon = get_right_side_of_colon(s);
     let result = right_side_of_colon.to_string();
 
-    return result;
+    result
 }
 
 fn parse_f32(s: &String) -> f32 {
-
     let result = s.parse::<f32>().unwrap();
 
-    return result;
+    result
 }
 
 fn parse_vec2(s: &String) -> Vec2 {
-    
-    if s.is_empty() {
-        return Vec2::new(0.0, 0.0);
-    }
+    let result = if s.is_empty() {
+        Vec2::new(0.0, 0.0)
+    } else {
+        let split = s.split(';').collect::<Vec<&str>>();
+        let value1 = split[0].trim();
+        let value2 = split[1].trim();
+        let result = Vec2::new(value1.parse::<f32>().unwrap(), value2.parse::<f32>().unwrap());
 
-    let split = s.split(';').collect::<Vec<&str>>();
-    let value1 = split[0].trim();
-    let value2 = split[1].trim();
-    let result = Vec2::new(value1.parse::<f32>().unwrap(), value2.parse::<f32>().unwrap());
+        result
+    };
 
-    return result;
+    result
+}
+
+fn parse_vec3(s: &String) -> Vec3 {
+    let result = if s.is_empty() {
+        Vec3::new(0.0, 0.0, 0.0)
+    } else {
+        let split = s.split(';').collect::<Vec<&str>>();
+        let value1 = split[0].trim();
+        let value2 = split[1].trim();
+        let value3 = split[2].trim();
+        let result = Vec3::new(value1.parse::<f32>().unwrap(), value2.parse::<f32>().unwrap(), value3.parse::<f32>().unwrap());
+
+        result
+    };
+
+    result
+}
+
+fn parse_vec4(s: &String) -> Vec4 {
+    let result = if s.is_empty() {
+        Vec4::new(0.0, 0.0, 0.0, 0.0)
+    } else {
+        let split = s.split(';').collect::<Vec<&str>>();
+        let value1 = split[0].trim();
+        let value2 = split[1].trim();
+        let value3 = split[2].trim();
+        let value4 = split[3].trim();
+        let result = Vec4::new(value1.parse::<f32>().unwrap(), value2.parse::<f32>().unwrap(), value3.parse::<f32>().unwrap(), value4.parse::<f32>().unwrap());
+
+        result
+    };
+
+    result
 }
 
 fn get_right_side_of_colon(s: String) -> String {
-    
     let split = s.split("//").collect::<Vec<&str>>();
     let split = split[0].split(':').collect::<Vec<&str>>();
-    let right_side_of_colon = split[1].trim();
+    let right_side_of_colon = split[1].trim().to_string();
 
-    return right_side_of_colon.to_string();
+    right_side_of_colon
 }
 
 fn instantiate_sprite_bundle(
@@ -510,7 +543,7 @@ fn instantiate_sprite_bundle(
         ..Default::default()
     };
 
-    return bundle;
+    bundle
 }
 
 fn instantiate_textbundle(
@@ -555,5 +588,5 @@ fn instantiate_textbundle(
         ..Default::default()
     };
 
-    return bundle;
+    bundle
 }
